@@ -208,7 +208,7 @@ public final class ToXmlGenerator
                 }
             }
         } catch (XMLStreamException e) {
-            StaxUtil.throwXmlAsIOException(e);
+            StaxUtil.throwAsGenerationException(e, this);
         }
     }
 
@@ -218,23 +218,6 @@ public final class ToXmlGenerator
     /**********************************************************
      */
 
-    /**
-     * Standard JSON indenter does not work well with XML, use
-     * default XML indenter instead.
-     *<p>
-     * !!! TODO: same as implementation in core 2.6; override may be
-     * removed from 2.8
-     */
-    @Override
-    public final JsonGenerator useDefaultPrettyPrinter()
-    {
-        // related to [dataformat-xml#136], need to verify:
-        if (_cfgPrettyPrinter != null) {
-            return this;
-        }
-        return setPrettyPrinter(_constructDefaultPrettyPrinter());
-    }
-
     @Override
     protected PrettyPrinter _constructDefaultPrettyPrinter() {
         return new DefaultXmlPrettyPrinter();
@@ -310,6 +293,14 @@ public final class ToXmlGenerator
         return this;
     }
 
+    @Override
+    public boolean canWriteFormattedNumbers() { return true; }
+
+    // @since 2.7.5
+    public boolean inRoot() {
+        return _writeContext.inRoot();
+    }
+
     /*
     /**********************************************************
     /* Extended API, access to some internal components
@@ -390,7 +381,7 @@ public final class ToXmlGenerator
                     _xmlWriter.writeStartElement(wrapperName.getNamespaceURI(), wrapperName.getLocalPart());
                 }
             } catch (XMLStreamException e) {
-                StaxUtil.throwXmlAsIOException(e);
+                StaxUtil.throwAsGenerationException(e, this);
             }
         }
         this.setNextName(wrappedName);
@@ -410,7 +401,7 @@ public final class ToXmlGenerator
                     _xmlWriter.writeEndElement();
                 }
             } catch (XMLStreamException e) {
-                StaxUtil.throwXmlAsIOException(e);
+                StaxUtil.throwAsGenerationException(e, this);
             }
         }
     }
@@ -454,7 +445,27 @@ public final class ToXmlGenerator
         writeFieldName(fieldName);
         writeString(value);
     }
+
+    // 03-Aug-2017, tatu: We could use this as mentioned in comment below BUT
+    //    since there is no counterpart for deserialization this will not
+    //    help us. Approaches that could/would help probably require different
+    //    handling...
+    //
+    //    See [dataformat-xml#4] for more context.
     
+    /*
+    // @since 2.9
+    public WritableTypeId writeTypePrefix(WritableTypeId typeIdDef) throws IOException
+    {
+        // 03-Aug-2017, tatu: Due to XML oddities, we do need to massage things
+        //     a bit: specifically, change WRAPPER_ARRAY into WRAPPER_OBJECT, always
+        if (typeIdDef.include == WritableTypeId.Inclusion.WRAPPER_ARRAY) {
+            typeIdDef.include = WritableTypeId.Inclusion.WRAPPER_OBJECT;
+        }
+        return super.writeTypePrefix(typeIdDef);
+    }
+    */
+
     /*
     /**********************************************************
     /* JsonGenerator output method implementations, structural
@@ -477,7 +488,7 @@ public final class ToXmlGenerator
     public final void writeEndArray() throws IOException
     {
         if (!_writeContext.inArray()) {
-            _reportError("Current context not an ARRAY but "+_writeContext.getTypeDesc());
+            _reportError("Current context not Array but "+_writeContext.typeDesc());
         }
         if (_cfgPrettyPrinter != null) {
             _cfgPrettyPrinter.writeEndArray(this, _writeContext.getEntryCount());
@@ -503,7 +514,7 @@ public final class ToXmlGenerator
     public final void writeEndObject() throws IOException
     {
         if (!_writeContext.inObject()) {
-            _reportError("Current context not an object but "+_writeContext.getTypeDesc());
+            _reportError("Current context not Object but "+_writeContext.typeDesc());
         }
         _writeContext = _writeContext.getParent();
         if (_cfgPrettyPrinter != null) {
@@ -526,7 +537,7 @@ public final class ToXmlGenerator
         try {
             _xmlWriter.writeStartElement(_nextName.getNamespaceURI(), _nextName.getLocalPart());
         } catch (XMLStreamException e) {
-            StaxUtil.throwXmlAsIOException(e);
+            StaxUtil.throwAsGenerationException(e, this);
         }
     }
     
@@ -550,7 +561,7 @@ public final class ToXmlGenerator
                 }
             }
         } catch (XMLStreamException e) {
-            StaxUtil.throwXmlAsIOException(e);
+            StaxUtil.throwAsGenerationException(e, this);
         }
     }
     
@@ -599,7 +610,7 @@ public final class ToXmlGenerator
                 _xmlWriter.writeEndElement();
             } 
         } catch (XMLStreamException e) {
-            StaxUtil.throwXmlAsIOException(e);
+            StaxUtil.throwAsGenerationException(e, this);
         }
     }    
     
@@ -634,7 +645,7 @@ public final class ToXmlGenerator
                 _xmlWriter.writeEndElement();
             }
         } catch (XMLStreamException e) {
-            StaxUtil.throwXmlAsIOException(e);
+            StaxUtil.throwAsGenerationException(e, this);
         }
     }
 
@@ -663,6 +674,82 @@ public final class ToXmlGenerator
     /**********************************************************
      */
 
+    @Override
+    public void writeRawValue(String text) throws IOException {
+        // [dataformat-xml#39]
+        if (_stax2Emulation) {
+            _reportUnimplementedStax2("writeRawValue");
+        }
+        try {
+            _verifyValueWrite("write raw value");
+            if (_nextName == null) {
+                handleMissingName();
+            }
+
+            if (_nextIsAttribute) {
+                _xmlWriter.writeAttribute(_nextName.getNamespaceURI(), _nextName.getLocalPart(), text);
+            } else {
+                _xmlWriter.writeStartElement(_nextName.getNamespaceURI(), _nextName.getLocalPart());
+                _xmlWriter.writeRaw(text);
+                _xmlWriter.writeEndElement();
+            }
+        } catch (XMLStreamException e) {
+            StaxUtil.throwAsGenerationException(e, this);
+        }
+    }
+
+    @Override
+    public void writeRawValue(String text, int offset, int len) throws IOException {
+        // [dataformat-xml#39]
+        if (_stax2Emulation) {
+            _reportUnimplementedStax2("writeRawValue");
+        }
+        try {
+            _verifyValueWrite("write raw value");
+            if (_nextName == null) {
+                handleMissingName();
+            }
+
+            if (_nextIsAttribute) {
+                _xmlWriter.writeAttribute(_nextName.getNamespaceURI(), _nextName.getLocalPart(), text.substring(offset, offset + len));
+            } else {
+                _xmlWriter.writeStartElement(_nextName.getNamespaceURI(), _nextName.getLocalPart());
+                _xmlWriter.writeRaw(text, offset, len);
+                _xmlWriter.writeEndElement();
+            }
+        } catch (XMLStreamException e) {
+            StaxUtil.throwAsGenerationException(e, this);
+        }
+    }
+
+    @Override
+    public void writeRawValue(char[] text, int offset, int len) throws IOException {
+        // [dataformat-xml#39]
+        if (_stax2Emulation) {
+            _reportUnimplementedStax2("writeRawValue");
+        }
+        _verifyValueWrite("write raw value");
+        if (_nextName == null) {
+            handleMissingName();
+        }
+        try {
+            if (_nextIsAttribute) {
+                _xmlWriter.writeAttribute(_nextName.getNamespaceURI(), _nextName.getLocalPart(), new String(text, offset, len));
+            } else {
+                _xmlWriter.writeStartElement(_nextName.getNamespaceURI(), _nextName.getLocalPart());
+                _xmlWriter.writeRaw(text, offset, len);
+                _xmlWriter.writeEndElement();
+            }
+        } catch (XMLStreamException e) {
+            StaxUtil.throwAsGenerationException(e, this);
+        }
+    }
+
+    @Override
+    public void writeRawValue(SerializableString text) throws IOException {
+        _reportUnsupportedOperation();
+    }
+
     @Override
     public void writeRaw(String text) throws IOException
     {
@@ -673,7 +760,7 @@ public final class ToXmlGenerator
         try {
             _xmlWriter.writeRaw(text);
         } catch (XMLStreamException e) {
-            StaxUtil.throwXmlAsIOException(e);
+            StaxUtil.throwAsGenerationException(e, this);
         }
     }
 
@@ -687,7 +774,7 @@ public final class ToXmlGenerator
         try {
             _xmlWriter.writeRaw(text, offset, len);
         } catch (XMLStreamException e) {
-            StaxUtil.throwXmlAsIOException(e);
+            StaxUtil.throwAsGenerationException(e, this);
         }
     }
 
@@ -701,7 +788,7 @@ public final class ToXmlGenerator
         try {
             _xmlWriter.writeRaw(text, offset, len);
         } catch (XMLStreamException e) {
-            StaxUtil.throwXmlAsIOException(e);
+            StaxUtil.throwAsGenerationException(e, this);
         }
     }
 
@@ -749,7 +836,7 @@ public final class ToXmlGenerator
                 }
             }
         } catch (XMLStreamException e) {
-            StaxUtil.throwXmlAsIOException(e);
+            StaxUtil.throwAsGenerationException(e, this);
         }
     }
 
@@ -797,7 +884,7 @@ public final class ToXmlGenerator
                 }
             }
         } catch (XMLStreamException e) {
-            StaxUtil.throwXmlAsIOException(e);
+            StaxUtil.throwAsGenerationException(e, this);
         }
     }
 
@@ -825,7 +912,7 @@ public final class ToXmlGenerator
                 }
             }
         } catch (XMLStreamException e) {
-            StaxUtil.throwXmlAsIOException(e);
+            StaxUtil.throwAsGenerationException(e, this);
         }
     }
 
@@ -854,7 +941,7 @@ public final class ToXmlGenerator
                 }
             }
         } catch (XMLStreamException e) {
-            StaxUtil.throwXmlAsIOException(e);
+            StaxUtil.throwAsGenerationException(e, this);
         }
     }
 
@@ -882,7 +969,7 @@ public final class ToXmlGenerator
                 }
             }
         } catch (XMLStreamException e) {
-            StaxUtil.throwXmlAsIOException(e);
+            StaxUtil.throwAsGenerationException(e, this);
         }
     }
 
@@ -910,7 +997,7 @@ public final class ToXmlGenerator
                 }
             }
         } catch (XMLStreamException e) {
-            StaxUtil.throwXmlAsIOException(e);
+            StaxUtil.throwAsGenerationException(e, this);
         }
     }
 
@@ -938,7 +1025,7 @@ public final class ToXmlGenerator
                 }
             }
         } catch (XMLStreamException e) {
-            StaxUtil.throwXmlAsIOException(e);
+            StaxUtil.throwAsGenerationException(e, this);
         }
     }
 
@@ -976,15 +1063,15 @@ public final class ToXmlGenerator
                 } else {
 	                _xmlWriter.writeStartElement(_nextName.getNamespaceURI(), _nextName.getLocalPart());
 	                if (usePlain) {
-	                    _xmlWriter.writeDecimal(dec);
-	                } else {
                          _xmlWriter.writeCharacters(dec.toPlainString());
+	                } else {
+                         _xmlWriter.writeDecimal(dec);
 	                }
 	                _xmlWriter.writeEndElement();
                 }
             }
         } catch (XMLStreamException e) {
-            StaxUtil.throwXmlAsIOException(e);
+            StaxUtil.throwAsGenerationException(e, this);
         }
     }
 
@@ -1017,7 +1104,7 @@ public final class ToXmlGenerator
                 }
             }
         } catch (XMLStreamException e) {
-            StaxUtil.throwXmlAsIOException(e);
+            StaxUtil.throwAsGenerationException(e, this);
         }
     }
 
@@ -1032,7 +1119,7 @@ public final class ToXmlGenerator
     /* Implementations, overrides for other methods
     /**********************************************************
      */
-    
+
     @Override
     protected final void _verifyValueWrite(String typeMsg) throws IOException
     {
@@ -1055,7 +1142,7 @@ public final class ToXmlGenerator
             try {
                 _xmlWriter.flush();
             } catch (XMLStreamException e) {
-                StaxUtil.throwXmlAsIOException(e);
+                StaxUtil.throwAsGenerationException(e, this);
             }
         }
     }
@@ -1070,7 +1157,12 @@ public final class ToXmlGenerator
         if (isEnabled(JsonGenerator.Feature.AUTO_CLOSE_JSON_CONTENT)) {
             try {
                 while (true) {
-                    JsonStreamContext ctxt = getOutputContext();
+		    /* 28-May-2016, tatu: To work around incompatibility introduced by
+		     *     `jackson-core` 2.8 where return type of `getOutputContext()`
+		     *     changed, let's do direct access here.
+		     */
+//                    JsonStreamContext ctxt = getOutputContext();
+		    JsonStreamContext ctxt = _writeContext;
                     if (ctxt.inArray()) {
                         writeEndArray();
                     } else if (ctxt.inObject()) {
@@ -1093,7 +1185,7 @@ public final class ToXmlGenerator
                 _xmlWriter.close();
             }
         } catch (XMLStreamException e) {
-            StaxUtil.throwXmlAsIOException(e);
+            StaxUtil.throwAsGenerationException(e, this);
         }
     }
 
